SystemOrganization addCategory: #KEGGeneratorInliningTest!

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:00'!
"(| defineMethod-:(" testcollectCsperatedByC "collect: operation separatedBy: separator "")
	notes-:(		Method Version: keg4.
				 Copyright Kjell E God June 19 1999
	licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details.
	Date of Last Revision: Date today tradePrint 20031222 20031212, February 02 2000, June 19 1999
	)-notes methodBody-:(
 (| let-:(""| params1 params2 |"")<---[ We can't use the lambda style( stick everything in a single BlockClosure )here because Generators often need localVariables.  And that is the case here.  They need a persistent state.  They can't get by on BlockClosure inputParameterVariables that are readOnly.  To force the lambda style onto the case of Generators we would have to use Arrays or some other holder Object instead of the localVariables and we just can't go that far.  It's getting just too Baroque then. ]
    letBody-:("					"( MT b:#collect:separatedBy: s:self ).""
	( params1 := Array new:( 1 ) ).
	( params2 := Array new:( 2 ) ).
	^(	( self	first:[ :x | params1 p1:( operation value:( x ) ) ] ""<---( we need this encapsulation to counter the inlineEmbeddedSequences or else if ( operation value:( x ) ) returns a sequence it will get taken apart and we don't want that. )""
			next:[ :x | params2 p1:( separator value:( x ) )p2:( operation value:( x ) ) ]
		) inlineEmbeddedSequences

	)""return:( 					MT e:#collect:separatedBy: s:self )"

    ")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(

	Return a new sequence based on self with seperator Objects
	inserted between the elements of self.  Just like the standard ANSI
	Collection based >>collect:separatedBy: .

 )-summary
 methodDescriptionDetail-:(

	PUBLIC-: ANSII-: Return a Generator with the following structure-:
	Let self = s = <s1 s2 s3... sN-1 sN>.
	operation is a <monadicValuable>. ( it responds to the >>value: message )
	seperator is a <monadicValuable>. ( it responds to the >>value: message )
	Let the returned Generator = r = <r1 r2 r3...R>.
	Let o(x) = ( operation value:( x ) ) . 
	Let s(x) = ( seperator value:( x ) ) .
	Then the sequence returned by self-:( this method ) is
		r = < o(s1) s(s1)   o(s2) s(s2)   o(s3) s(s3)...o(sN-1)    s(sN-1) o(sN) >.
	Thus if s = < s1 > =then=> r = < o(s1) >.
	Thus if s = < s1 s2 > =then=> r = < o(s1) s(s1) o(s2) >.
	(( If self has just one element( self isSingleton )THEN-: self is returned. ))
	(( o(x) is the results of evaluating ( operation value:( x ) ) ))

	(( a sequence is any Object in some variable-:( s )that cases
	   the expression ( s isSequence ) to return-:( true )when it is evaluated.
	   All KEGGenerators will cause ( s isSequence ) to return-:( true )when
	   they are put into the some Smalltalk variable-:(  s ) and then ( s isSequence )
	   is evaluated .    Here-:( s )is a box with a Smalltalk variable in it.
	   A Smalltalk variable is also a box.  So-:( s )here would be a box with
	   a box inside of it.   You can't really do that in Smalltalk but you
	   could in Lisp.  Please see-:(  rant-:( 1 )  )below for more useless tripe
	   about it. ))

 )-detail
 usage-:(
	usage
 )-usage
 todos-:(
	todos
 )-todos
 changes-:(

	(| KEG5-:( Changes onDate-:(  ) madeChanges-:( ... )-madeChanges )-chg )-keg5

	(| KEG4-:( 
		Changes onDate-:( 20031222 ) madeChanges-:(
			Rewrote self using a different more code simple algorithm that starts
			over from scratch using the KEGGenerator>>first:next: method.
			This simplified it much and the debugging was very quick taking
			just a few minutes of tests.  I had to make this change because
			the previous mess just was not working and seemed way more
			complex than it needed to be and was just going to take too
			long to debug.
		)-madechanges 
	)-chg )-KEG4

	(| KEG3-:( 
		Changes onDate-:( 20031212 ) madeChanges-:(
			Rewrote self into the more recent Lispy style now used
			and added the call to-:( >>asCanBackUp1 )to handle the case where
			self has just one element in it better.  The rest of the
			system may be dependent on this bug in the case where
			self has just one element so there may be some random
			bugs coming up because of it but i hope not.
		)-madechanges 
	)-chg )-keg3

	(| KEG2-:( 
		Changes onDate-:( February 02 2000 ) madeChanges-:(
			KEG2: made a totally new method body that actually works
			like the comment above says.
		)-madechanges 
	)-chg )-keg2

 )-changes
 testCases-:( "

	( self assert:( 
		(
		(
		( #( 1 ) asGenerator 
		) collect:( [ :x | x ] )separatedBy:[ :na | '-' ]
		) asArray = #(1)
		)
		)description:( 'Test 1 >>collect:seperatedBy:' ) ).

	( self assert:( 	
		(
		(
		( #( 1 2 ) asGenerator 
		) collect:( [ :x | x ] )separatedBy:[ :na | '-' ]
		) asArray = #(1 '-' 2)
		)
		)description:( 'Test 2 >>collect:seperatedBy:' ) ).

	( self assert:( 
		(
		(
		( #( 1 2 3 4 ) asGenerator 
		) collect:( nil )separatedBy:[ :na | '-' ] "<---[        nil = [ :x | x ]        ]"
		) asArray = #(1 '-' 2 '-' 3 '-' 4)
		)
		)description:( 'Test 3 >>collect:seperatedBy:' ) ).

 " )-testCases
 alternateCode-:(

	| params gen sepBlk aKEGFooArray first backUp n |

	( first := true ).""<-----------------------------------------------------( to handle the case where self isSingleton. )""
	( gen := ( params := Array with:( 1 )with:( 1 ) )asGenerator ).
	( aKEGFooArray := KEGFooArray new ). ""<-------------------( to get rid of the prefix seperator that would otherwise always results. )""
	( sepBlk := [ :x | sepBlk := separator . aKEGFooArray ] ).
	""<---( lineA code position )""
	( 
		( backUp := self asCanBackUp1 ""<---( EXAMPLE-: Here is an example of how a Generator can be turned into a KEGSimpleCanBackup1 as a( code simple )and probably fast execution way to look ahead 1 into a Generator without using the >>lookAhead1: method.  ( [_]:[]:WK SMALLTALK-: Dolphin Smalltalk-: In fact >>lookAhead1: could probably be looked at in future to see if it could/should be reImplemented in terms of KEGSimpleCanBackup1 -trd . ) )""
		) atEachGenerate:[ :x |
			(
			(	( first )						and:[
				(
				( ( n := backUp next )isNil ""<---( If this returns true then self has only one element in it.  So we don't want the extra prefixed seperator that would otherwise result. )""
				) doYourself:[ ( first := false ).( backUp next:( n ) ) ]""<---( Here we clean up the usually errant ( n := backUp next ) we just did by putting-:( n )back onto the front of the backUp Generator. We set the next element of backUp to-:( n ).
 Which causes backUp essentially to back up one.  So now the next evaluation of ( backUp next ) will return this-:( n )we just got here again.  Just as if we had never done the ( backUp next ) here at all.  This shows one of the convenient things you can do with KEGGenerators.  You can look ahead into them or you can just grab the next element in them without waiting to go through the usual BlockClosure iteration channels.  You can just grab it from inside of one of the iteration BlockClosures as we have done here.  This can often be the most natural thing to do.  And can be a very powerful functionality to have.  It often makes the most sense and so makes the resulting code the most code minimal.   >>doYouself: just evaluates the BlockClosure and returns self so it is similar to the standard Smalltalk >>yourself .  that self returned in this case is the Boolean result coming from >>isNil above. )""
				)							]""-and: ""
			) ifTrue:[ Array with:( x ) ]""<---( Return     x     the only element of self wrapped in an Array ready to be wrapped in a singleton KEGGenerator by the Generator G previously created by KEGGenerator>>atEachGenerate: .  ( G next ) has somewhere been started up and is now being evaluated.  We are somewhere in the middle of the evaluation of ( G next ) right here.  We cannot just check to see if self has one element in it up at( lineA code position )above and then just return self.  Because of the KEGAbstractGenerator>>on: method which allows the basis sequence of G to change after G has been created and returned by self-:( this method )( and thus self changes as well since G has self embedded inside of it and the basis of self is also the basis of G and any Generators based on G ).  The basis sequence is the fundamental headwaters sequence self and thus G is generating over and when it changes then self changes as a sequence( if previous to the >>on: message send finishing up and returning its results to its sender we have self = < s1 s2 s3 ... > then afterwards self might be some other sequence and then we would have something like self = < a1 a2 a3 ... > where at least one of the-:( a's )is different from the cooresponding-:( s ).  The length of self might even change.  self might even become empty. ).  But the way self generates( iterates ) would not change( because of an >>on: message going off on it or some Generator based on it( like G ) ).  The way self iterates is defined by the method that created self.  Just in the same way as we have here where the way G iterates is defined by self-:( this method ) which is the method that created-:( G ). )""
			  ifFalse:[
				(
				( params ""<----------------------------------------------( Generators can reuse parameter passing Objects like-:( param )( used here ).  They can use the same one for each iteration pass of a Generator G during iteration doing messages like KEGGenerator>>asArray( like if ( G asArray ) were evaluated. ).  This kind of iteration parameter Object reuse is not possible using Collections.  Because you would then need a different new param Array for each new iteration of the Collection.  By the way The iteration functionality ie-:( >>collect: >>select: >>reject: >>detect: ... etc )of the KEGGenerator Class is a superSet of the iteration functionality of the Collection Class.  By a long shot.  It has way bigger functionality.  To the point where i find it hard to do anything without Generators.  I use them in just about everything.  Instead of and together with Collections.   Most Generators are at bottom based on a Collection.  Or a Stream.  Some Generators create their data from scratch like the one returned by KEGGenerator>>counter . )""
				) p1:( sepBlk value:( x ) )""Seperator""
				  p2:( operation value:( x ) )""NonSeperator""   ""<---( ( a p1:x p2:y ) is exactly like ( a at:1put:x ; at:2put:y ; yourself ).  It's just a little prettier. )""
				).
				gen ]""<---( Return gen to be iterated over by >>atEachGenerate: )""
			) ] 
		) reject:[ :x | x == aKEGFooArray ]""<---( This gets rid of the leading seperator.  Because as the standard specification and the one below states we don't want a prefix seperator. )""


























	(----------------------------The Old Body Of The Method Begins Here--------------------------
	sep := [ :e | sep := separator ].	""<---( Dolphin: Switch to the real separator after first eval. KEG: very sneaky. )""
	answer := self collect:[ :e | ""<---( KEG: Switch do: to collect: )""
		sep value: e .
		operation value: e 
	] .
	===================The Old Body Of The Method Ends Here==================)

 )-alternateCode
 designs-:(
	( DESIGN subject-:( aPackageName )of-:(
	)-of )-design
 )-designs
 rants-:(
	rants
 )-rants )-defineMethod"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:01'!
"(| defineMethod-:(" testconcat "concat "")
	notes-:(		Method Version: keg1.
				 Copyright Kjell E God 20030103
	licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details.
	Date of Last Revision: Date today tradePrint 20030103
	)-notes methodBody-:(
 (| let-:("		")
    letBody-:("					"( MT b:#concat s:self ).""
	^(
		( self inlineEmbeddedSequences )

	)""return:( 					MT e:#concat s:self )"

    ")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(

	Concatentate all the subSequences inside self together.

 )-summary
 methodDescriptionDetail-:(

	Here we just rename the KEGGenerator>>inlineEmbeddedSequences method
	because the purpose of self-:( this method ) is mainly just for documentation
	purposes.  In order to list all the real concatenation methods.
	This functionality is actually done elsewhere in the methods-:(
		KEGGenerator>>,
		KEGGenerator>>atEachGenerate:
		KEGGenerator>>inlineEmbeddedSequences
		KEGGenerator>>inlineEmbeddedSequencesBreadthFirst:
		KEGGenerator>>inlineEmbeddedSequencesDeep
		KEGGenerator>>inlineEmbeddedSequencesDeep:
		KEGGenerator>>asString <-----------( in case the contents of self are Strings or Characters )
		KEGStringGenerator>>asString <---( in case self is a KEGStringGenerator and the contents
											of self are Objects which can >>displayOn: convert
											themselves into Strings ( Strings themselves are such ) )
	)-methods
	but is redone here as a place where such concatenation methods
	and techniques can be listed.  In one central place.  There may be
	others out there that are not listed here yet.

 )-detail
 usage-:(

	See the methods-:(
		File class>>splitPathName:
	)for an example of how to get the concatenated pathName out of the
	message-:( File class>>splitPathFrom: ).
	Also see all the references( sendersOf: ) the concatenation methods listed above
	to see other ways concatenation can be done.

 )-usage
 todos-:(
	todos
 )-todos
 changes-:(

	(| KEG2-:( Changes onDate-:( ... ) madeChanges-:( ... )-madechanges )-chg )-keg2

 )-changes
 testCases-:( "

	( self assert:( 
		#( (  )(  )(  ) ) asGenerator concat asArray = #(  )
		)description:( '' ) ).

	( self assert:( 
		#( ( 1 )(  )( 7 ) ) asGenerator concat asArray = #( 1 7 )
		)description:( '' ) ).

	( self assert:( 
		#( ( 1 2 3 )(  )( 7 8 9 ) ) asGenerator concat asArray = #( 1 2 3 7 8 9 )
		)description:( '' ) ).

	( self assert:(  
		#( ( 1 2 3 )( 4 )( 7 8 9 ) ) asGenerator concat asArray = #( 1 2 3 4 7 8 9 )
		)description:( '' ) ).

	( self assert:( 
		#( ( 1 2 3 )( 4 5 6 )( 7 8 9 ) ) asGenerator concat asArray = #( 1 2 3 4 5 6 7 8 9 )
		)description:( '' ) ).

 " )-testCases
 rants-:(
	rantsrantsrants rantsrants
 )-rants )-defineMethod"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/11/2013 12:30'!
"(| defineMethod-:(" testdisperseC")
	notes-:(		Method Version: keg3.
				 Copyright Kjell E God Mar  9 1999
	copyLefted as GPL Open Source: Please execute ( Object copyLeft ) for details.
	Date of Last Revision: Date today tradePrint 20040204 , ? , Mar  9 1999
	<< OWNER-: Kjell E God SOFTWARE-: SOSITPlatform VERSION-: 3 >>
	)-notes methodBody-:(
 (| let-:(""| answer next return amDispersing disperseBlk gen p streamGen |"")
    letBody-:(""					""( MT b:#disperse: s:self ).""

  ( streamGen := KEGGenerator streamGenerator ). ""<-----( KEG3 )""
  ( p := [ :g | 
	(
	( amDispersing
	) ifTrue:[ gen := gen , ( g asGenerator ) ] ""<---( KEG2-: Now multiple ( p value: ... ) ops can be done during a single ( returnGen next ) operation.  Where returnGen is   Thus making ( p value:( ... ) ) not unlike the Stream >>nextPut: and >>nextPutAll: methods. 		( [_]:[]:WK SMALLTALK-: Dolphin Smalltalk-: In future Stream based speed optimizations could probably be done here in case the g's are based on Streams which they probably are.  Something like-:((((        gen isOnAStream ifTrue:[ gen generator nextPutAll:( g asGenerator if:[ :sq | sq isOnAStream ]isTrue:[ :sq | sq generator collection ]isFalse:[ :sq | sq ] ) ]ifFalse:[ gen := gen , ( g asGenerator ) ]       ))))but since I don't need this stuff as of yet I am blowing it off here by not including it.  So it's saved for the future . ) )""
	  ifFalse:[ gen := g asStreamGeneratorOn:( streamGen ) ] 
	).( gen reset ).( amDispersing := true )
  ] ) .

  ^( self abstractGenerator new
	resetBlock:[ 
		( amDispersing := false ). 
		( ( next := self first )
		) ifNotNil:[ next := transformer value:( p )value:( next ) ] ]
	nextBlock:( 
		disperseBlk := [
		     ( amDispersing
		     ) ifTrue:[ ( return := gen next ).
				( return
				) ifNil:[ 
					( amDispersing := false ).
					( ""...............................""
					( ( next := self next )
					) ifNotNil:[ next := transformer value:( p )value:( next ) ] 
					). 
					( disperseBlk value ) ]
				  ifNotNil:[ return ] ]
		       ifFalse:[ 
				( return := next ).
				( ""....................""
				( ( next := self next )
				) ifNotNil:[ next := transformer value:( p )value:( next ) ]
				).
				return ]
		]""-disperseBlk "" )

  )""return:( 					MT e:#disperse: s:self )""

    "")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(
	
	transformer is a diadicValuable ( ie: it is anything that respondsTo:( #value:value: ) )
	like a two input parameter or two input port BlockClosure.

 )-summary
 methodDescriptionDetail-:(

	I will just make an example-:(

	( r :=
	( aGenerator disperse:[ :p :x | 
		( x isSequence 
		) ifTrue:[ p value:( x ) ]
		  ifFalse:[ x ]
	  ]
	))

	).  In the above example p is a BlockClosure or some other monadicValuable.
	You can pass a sequence into it or any other kind of Object that responds to
	>>asGenerator ( which is any Object ) and that sequence will be spliced into the
	sequence returned by the above >>disperse: message.
	So first you do the ( p value:( someSequence ) ) and then you exit the transformer
	Block and the someSequence would be inserted into the sequence ( r ) at that point.
	You can also perhaps do stuff like-:(

	( r :=
	( aGenerator disperse:[ :p :x | 
		( x isSequence 
		) ifTrue:[ ( p value:( x ) ).( p value:#( 1 2 3 4 ) ) ]
		  ifFalse:[ x ]
	  ]
	))

	) where first x is inserted into the ( r ) sequence and then the #( 1 2 3 4 )
	sequence is inserted into ( r ).

	Please see the test cases below.

 )-detail
 usage-:(
	usage
 )-usage
 todos-:(
	todos
 )-todos
 changes-:(
	(| KEG4-:( Changes onDate-:( ... ) madeChanges-:( ... )-madechanges )-chg )-keg4
	or
	(| KEG4-:( Changes onDate-:( ... ) theOldCode-:( ... ) wasReplacedBy-:( ... ) )-chg )-keg4
	(| KEG4-:( Changes onDate-:( 20040204 ) madeChanges-:( 
		Added streamGen stuff.
	)-madechanges )-chg )-keg4
 )-changes
 testCases-:( "

	| g |

	( self assert:(
		( #(  ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #(  )
		) description:( '( #(  ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #(  )' ) ).

	( self assert:(
		( #( (  ) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #(  )
		) description:( '( #(  ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #(  )' ) ).

	( self assert:(
		( #( ( (  ) ) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( (  ) )
		) description:( '( #(  ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #(  )' ) ).

	( self assert:(
		( #( 1 2 3 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 )
		) description:( '( #( 1 2 3 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 )' ) ).

	( self assert:(
		( #( (1 2 3) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 )
		) description:( '( #( (1 2 3) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 )' ) ).

	( self assert:(
		( #( 1 (2 3 4) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 )
		) description:( '( #( 1 (2 3 4) ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 )' ) ).

	( self assert:(
		( #( (1 2 3) 4 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 )
		) description:( '( #( (1 2 3) 4 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 )' ) ).

	( self assert:(
		( #( (1 ( 2 3 ) ) 4 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 ( 2 3 ) 4 )
		) description:( '( #( (1 ( 2 3 ) ) 4 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 ( 2 3 ) 4 )' ) ).

	( self assert:(
		( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 7 8 9 )
		) description:( '( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ p value:( x ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 7 8 9 )' ) ).

	( self assert:(
		( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )
		) description:( '( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | ( x isSequence )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )' ) ).

	( self assert:(
		( #( 1 2 3 4 5 6 7 8 9 ) asGenerator disperse:[ :p :x | ( x = 6 )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )
		) description:( '( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | ( x = 6 )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )' ) ).

	"<---[ This shows how you can look ahead if you don't care about how g gets repositioned.  If you do care then look at the comments in KEGGenerator>>nowNext:into: ]"
	( self assert:(
		( ( g := #( 1 2 3 4 5 6 7 8 9 ) asGenerator )disperse:[ :p :x | ( ( x = 6 )and:[ ( g nowNext: 2 into:( ( Array new: 2 ) writeStream ) ) contentsKEG = #( 7 8 ) ] )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 7 8 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )
		) description:( '( ( g := #( 1 2 3 4 5 6 7 8 9 ) asGenerator )disperse:[ :p :x | ( ( x = 6 )and:[ ( g nowNext: 2 into:( ( Array new: 2 ) writeStream ) ) contentsKEG = #( 7 8 ) ] )ifTrue:[ ( p value:( x ) ).( p value:#( 6 5 4 7 8 ) ) ]ifFalse:[ x ] ] ) asArray = #( 1 2 3 4 5 6 6 5 4 7 8 9 )' ) ).

	( self assert:(
		( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | p value:( x ) ] ) asArray = #( 1 2 3 4 5 6 7 8 9 )
		) description:( '( #( 1 2 3 ( 4 5 6 ) 7 8 9 ) asGenerator disperse:[ :p :x | p value:( x ) ] ) asArray = #( 1 2 3 4 5 6 7 8 9 )' ) ).

 " )-testCases
 alternateCode-:(
 )-alternateCode
 designs-:(
	( DESIGN subject-:( aPackageName )of-:(
	)-of )-design
 )-designs
 rants-:(
	
	<< See Zoomer FILE: c:\teleagnt\text\paperfil.zmr AT: !!, >> 
	or evaluate the following-: (( KEGHelpString helpStringAt: 6 ) notify: nil ) or 
	Get Text Pointed At By Selection on << See HelpString AT-: 6 >>

 )-rants )-defineMethod"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:07'!
testinlineEmbeddedSequences "inlineEmbeddedSequences"

	"Method Version: keg1.  "
	" Copyright Kjell E God Dec 15 1999"
	"licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details."
	"Date of Last Revision: Date today Dec 15 1999""
	| answer streamGen array |

						""MT b:#inlineEmbeddedSequences s:self.""

	array := ( streamGen := KEGGenerator streamGenerator ) generator collection . ""<---[ KEG2: This means we don't have to allocate a new Generator for each nonGenerator x. ]""
	answer := self atEachGenerate:[ :x | x asStreamGeneratorOn:( streamGen on: array ) ] .""<---[ Reprime the streamGen in case the previous x is a Collection other than Array.  We need Array to be in there now in case x is a String.  We don't want Strings to be broken up into a sequence of Characters. ]""

						""MT e:#inlineEmbeddedSequences s:self.""
	^answer .

	""Return a KEGAbstractGenerator that Concatenates the top level subSequences in self.  
	Example: <<1 2 3> 4 <5> <6 7 8> 9 10 > becomes <1 2 3 4 5 6 7 8 9 10>.
	Example: <<1 2 3> 4 <5> <<6> 7 8> 9 10 > becomes <1 2 3 4 5 <6> 7 8 9 10>.
	KEG2: Added the use of the new more self documenting >>streamGenerator."

	"Tests-: "

	( self assert:(
		#(  ) asGenerator inlineEmbeddedSequences asArray = #(  )
		) description:( '#(  ) asGenerator inlineEmbeddedSequences asArray = #(  )' ) ).

	( self assert:(
		#( (  ) ) asGenerator inlineEmbeddedSequences asArray = #(  )
		) description:( '#( (  ) ) asGenerator inlineEmbeddedSequences asArray = #(  )' ) ).

	( self assert:(
		#( ( (  ) ) ) asGenerator inlineEmbeddedSequences asArray = #( (  ) )
		) description:( '#( ( (  ) ) ) asGenerator inlineEmbeddedSequences asArray = #( (  ) )' ) ).

	( self assert:(
		#( 1 ) asGenerator inlineEmbeddedSequences asArray = #( 1 )
		) description:( '#( 1 ) asGenerator inlineEmbeddedSequences asArray = #( 1 )' ) ).

	( self assert:(
		#( 1 2 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 )
		) description:( '#( 1 2 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 )' ) ).

	( self assert:(
		#( 1 2 3 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 )
		) description:( '#( 1 2 3 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 )' ) ).

	( self assert:(
		'123' asGenerator inlineEmbeddedSequences asArray = #( '123' )
		) description:( '''123'' asGenerator inlineEmbeddedSequences asArray = #( ''123'' )' ) ).

	( self assert:(
		#( '123' ) asGenerator inlineEmbeddedSequences asArray = #( '123' )
		) description:( '#( ''123'' ) asGenerator inlineEmbeddedSequences asArray = #( ''123'' )' ) ).

	( self assert:(
		#( ( '123' ) ) asGenerator inlineEmbeddedSequences asArray = #( '123' )
		) description:( '#( ( ''123'' ) ) asGenerator inlineEmbeddedSequences asArray = #( ''123'' )' ) ).

	( self assert:(
		#( ( ( '123' ) ) ) asGenerator inlineEmbeddedSequences asArray = #( ( '123' )  )
		) description:( '#( ( ( ''123'' ) ) ) asGenerator inlineEmbeddedSequences asArray = #( ( ''123'' )  )' ) ).

	( self assert:(
		#( (1 2 3) '(444)' (5) (6 7 8) 9 10 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		) description:( '#( (1 2 3) ''(444)'' (5) (6 7 8) 9 10 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 ''(444)'' 5 6 7 8 9 10 )' ) ).
	
	( self assert:(
		#( (1 2 3) '(444)' (5) (6 (7) 8) 9 10 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 '(444)' 5 6 (7) 8 9 10 )
		) description:( '#( (1 2 3) ''(444)'' (5) (6 (7) 8) 9 10 ) asGenerator inlineEmbeddedSequences asArray = #( 1 2 3 ''(444)'' 5 6 (7) 8 9 10 )' ) ).





"--------------------------Old-Code-Replaced-By-KEG2---------------------------------
streamGen := ( oc := OrderedCollection with: 1 ) asGenerator . <---( This means we don't have to allocate a new Generator for each nonGenerator x. )
=============Old-Code-Replaced-By-KEG2=================
"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:08'!
testinlineEmbeddedSequencesDeep "inlineEmbeddedSequencesDeep"

	"Method Version: keg2.  "
	" Copyright Kjell E God 20010726"
	"licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details."
	"Date of Last Revision: Date today tradePrint 20030306, 20010726""
	| answer streamGen array g emptyGen |

						""MT b:#inlineEmbeddedSequencesDeep s:self.""

	array := ( streamGen := KEGGenerator streamGenerator ) generator collection . ""<---( This means we don't have to allocate a new Generator for each nonGenerator x. )""
	( emptyGen := ( self class empty ) ).
	answer := self atEachGenerate:[ :x | 
		( g := ( x asStreamGeneratorOn:( streamGen on: array ) ) ).""<---( Reprime the streamGen in case the previous x is a Collection other than OrderedCollection.  We need an Array to be in there in case x is a String. )""
		( x isSequence
		) ifTrue:[ ( x isEmpty )ifTrue:[ emptyGen ]ifFalse:[ g inlineEmbeddedSequencesDeep ] ]
		  ifFalse:[ g ]
	] .

						""MT e:#inlineEmbeddedSequencesDeep s:self.""
	^answer .

	""Return a KEGAbstractGenerator which Concatenates all the embedded subSequences in self
	when it is iterated like with >>asArray or >>asOrderedCollection or >>iterate or etc.
	Example: <<1 2 3> 4 <5> <6 7 8> 9 10 > becomes <1 2 3 4 5 6 7 8 9 10>.
	Example: <<1 2 3> 4 <5> <<6> 7 8> 9 10 > becomes <1 2 3 4 5 6 7 8 9 10>."

	"(|KEG2-: <D:20030306> added the isEmpty and g to get rid of the infinite loop. )

	Tests-: "

	( self assert:(
		#(  ) asGenerator inlineEmbeddedSequencesDeep asArray = #(  )
		) description:( '#(  ) asGenerator inlineEmbeddedSequencesDeep asArray = #(  )' ) ).

	( self assert:(
		#( ( (  ) ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #(  )
		) description:( '#( ( (  ) ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #(  )' ) ).

	( self assert:(
		#( ( ( 1 ) ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 )
		) description:( '#( ( ( 1 ) ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 )' ) ).

	( self assert:(
		#( 1 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 )
		) description:( '#( 1 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 )' ) ).

	( self assert:(
		#( 1 2 3 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 )
		) description:( '#( 1 2 3 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 )' ) ).

	( self assert:(
		#( 1 ( ( 2 ( 3 ( ( 4 ) 5 ) ) ) 6 ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )
		) description:( '#( 1 ( ( 2 ( 3 ( ( 4 ) 5 ) ) ) 6 ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )' ) ).

	( self assert:(
		#( 1 2 3 4 ( 5 6 ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )
		) description:( '#( 1 2 3 4 ( 5 6 ) ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )' ) ).

	( self assert:(
		#( ( 1 2 3 ) 4 5 6 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )
		) description:( '#( ( 1 2 3 ) 4 5 6 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 4 5 6 )' ) ).

	( self assert:(
		#( (1 2 3) '(444)' (5) (6 7 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		) description:( '#( (1 2 3) ''(444)'' (5) (6 7 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 ''(444)'' 5 6 7 8 9 10 )' ) ).
	
	( self assert:(
		#( 1 2 3 '(444)' (5) ((6) 7 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray	= #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		) description:( '#( 1 2 3 ''(444)'' (5) ((6) 7 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray	= #( 1 2 3 ''(444)'' 5 6 7 8 9 10 )' ) ).

	( self assert:(
		#( (1 2 3) '(444)' (5) (((6) 7) 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		) description:( '#( (1 2 3) ''(444)'' (5) (((6) 7) 8) 9 10 ) asGenerator inlineEmbeddedSequencesDeep asArray = #( 1 2 3 ''(444)'' 5 6 7 8 9 10 )' ) ).



"--------------------------Old-Code-Replaced-By-KEG2---------------------------------
streamGen := ( oc := OrderedCollection with: 1 ) asGenerator . <---( This means we don't have to allocate a new Generator for each nonGenerator x. )
=============Old-Code-Replaced-By-KEG2=================
"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:09'!
"(| defineMethod-:(" testinlineEmbeddedSequencesDeepC "inlineEmbeddedSequencesDeep: explodeBlock "")
	notes-:(		Method Version: keg1.
				 Copyright Kjell E God 20010726
	copyLefted as GPL Open Source: Please execute ( Object copyLeft ) for details.
	Date of Last Revision: Date today tradePrint 20010726
	)-notes methodBody-:(
 (| let-:(""| emptySeq streamGen1 array1 streamGen2 abstractGen2 inlineGen array2 concatGen
		param p1 p2 didEmpty paramInput atEachGen |"")
    letBody-:(""					""( MT b:#inlineEmbeddedSequencesDeep: s:self ).""

	( self isEmpty ifTrue:[ ^self ] ).
	( emptySeq := #(  ) ).
	( array1 := ( streamGen1 := KEGGenerator streamGenerator ) generator collection ). ""<------------( This means we don't have to allocate a new Generator for each time around the atEachGenerate: loop. )""
	( array2 := ( streamGen2 := KEGGenerator streamGenerator ) generator collection ). ""<------------( This means we don't have to allocate a new Generator for each time around the atEachGenerate: loop. )""
	( abstractGen2 := KEGAbstractGenerator new resetBlock:[ inlineGen reset ]nextBlock:[ inlineGen next ] ). ""<---( This means we don't have to allocate a new Generator for each time around the atEachGenerate: loop. )""
	( atEachGen := concatGen := ( streamGen1, abstractGen2 ) ). ""<---------------------------------------------------( This is a machine that is the sequence of the contents of streamGen1 followed by the contents of abstractGen2.  This concatGen is set up above and below to be ( param at:1 ) followed by a depthfirst traversal of ( param at:2 ).  The sequence concatGen can create a Collection of the concatGen sequence, or iterate over the concatGen sequence depending on what you want depending on if you use >>asOrderedCollection or >>do: or >>inject:into: or >>asString etc on the answer returned by self( this method ). Since we create concatGen outside of the atEachGenerate: loop below it means we don't have to allocate a new concatenation Generator for each time around the atEachGenerate: loop. )""
	( paramInput := Array new:( 2 ) ).
	( didEmpty:=false ).

	^(
		self atEachGenerate:[ :x | ""<------------------------------------------------------------------------( Here is the recursive Generator allocation.  Thus the Generators inlining the tree structure of self will mirror the tree structure and be as big as the tree structure.  There are ways to minimize this memory allocation so that no more Generators need be allocated than the tree is tall, but those ways are not being used here in order to simplify things.  And so make them more robust. )""
			( param := explodeBlock value:
				( ( x isSequence )
					ifTrue:[ paramInput at:1put:( nil );at:2put:( x ) ; yourself ]
					ifFalse:[ paramInput at:1put:( x );at:2put:( nil ) ; yourself ]""<------------------------------( This will cause inlineGen to become nil the empty sequence. )""
				)
			).
			""<-----------------------------------------------------------------------------------------------( The concatGen is already created like a machine above.  It has two pipes going into it into which we can dump sequences.  streamGen1 and abstractGen2.  What ever we stick into streamGen1 and abstractGen2 will be iterated over by concatGen.  And comcatGen does a depth first traversal of the contents of ( param at:2 ).  So we can stick different things into these two pipes and each time concatGen will be like a machine and iterate over them.  But it is lazy so it will not iterate over them until it is asked to.  It will just sit there with streamGen1 and abstractGen2 in its mouth.  It will not eat them until it is asked to iterate via >>asOrderedCollection or >>do: or >>inject:into: or >>iterate or one of the other Generator iteration methods. )""
			(
			( p1 := ( param at:1 ) ) isNil
				ifTrue:[ ( streamGen1 on:( emptySeq ) ).( didEmpty:=true ) ]
				ifFalse:[ 
					""( oc1 removeAll ). <--------------------------------------------------------------( PLEASE-NOTE-: Since array1 always has exactly 1 element at:1 then we don't have to do any PositionableStream>>initialize ie: Stream resizing to fit the OrderedCollection which has changed size.  Because it hasn't changed size.  And now it's an Array and not an OrderedCollection anymore. )""
					( didEmpty ifTrue:[ ( didEmpty:=false ).( streamGen1 on:( array1 ) ) ] ).""<---( array1 should satisfy ( array1 size = 1 ) here and always. )""
					( array1 at:1put:( p1 ) ) ] ""<---------------------------------------------------------( Here is where we set the contents of streamGen1 to be the value of the tree node x. )""
			).
			""( atEachGen := concatGen ).<-------------------------------------------------------------( atEachGen only actually has any meaning when the alternateCode-: below is used instead of self-:( this statement ) and the next adjacent statement.  It is decided that we are going to use the concatGen in all cases so as to simplify the code here.  We are thus assuming the speedup we might get using the alternateCode-: below is negligable. )""
			( inlineGen := 
				( 
					( param at:2 )asStreamGeneratorOn:( streamGen2 on:( array2 ) ) 
				)	inlineEmbeddedSequencesDeep:( explodeBlock )""<-------------------------( Here is where we set the contents of abstractGen2 to be the depth first traversal of the children of tree node x.  The >>on: message Reprimes the streamGen2 in case the previous time around the atEachGenerate: loop ( param at:2 ) was a Collection other than Array.  We need Array array2 to be in there now as a speed optimization in case ( param at:2 ) is a String. )""
			).
			""<------------------------------------------------------------------------------------------( Now that we have dumped value1=( param at:1 ) and sequence2 = ( param at:2 ) into the concatGen all we need to do now is return concatGen which is the sequence of value1 followed by the depth first traversal of sequence2.  This concatGen will be iterated over by atEachGenerate: . )""
			atEachGen
		]

	)""return:( 					MT e:#inlineEmbeddedSequencesDeep: s:self )""

    "")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(

	Concatenate all the embedded subSequences in self into a single linear
	sequence without any subSequences.
	( use the <PopUpMenu>=> <M>( Manipulate... )=>
		<T>( Toggle the =( Word Wrap )= )
	command in self-:( this text pane ) to toggle the word wrap so as to view
	the code above or the comments above as needed. )

 )-summary
 methodDescriptionDetail-:(

	explodeBlock is expected to
	take a single Array parameter input
	and to return an Array of size at least 2 of the form-:(
		Array( anObject1 aSequence2 )
	)
	which doesn't have to be the same Array P as was input into explodeBlock
	but P is provided as a convenience so the user method will not have to allocate
	one.
	As a convenience the following will be true of P -:(
		( P p1 = nil ) => ( P p2 isSequence )=true
		( P p2 = nil ) => ( P p1 isSequence )=false
	)
	So you can use isNil tests instead of the isSequence tests if you want.
	. . . . .
	Above anObject1 and aSequence2 can be any Objects( aSequence2 doesn't
	actually have to be a sequence.  Where a sequence is any s which makes
	( s isSequence ) return true.
	There are speed optimizations in the cases that either anObject1 or
	aSequence2 are nil.

	So why use self( this method )?
	In what cases can it be useful?  Suppose you have a tree structure where each
	node has a value and some children.  So you can do ( aNode value ) and 
	( aNode children ) and get the value and
	the children.  Then you could do-:
	---------------------
	answer := ( tree asGenerator )
		inlineEmbeddedSequencesDeep:[ :p |
			p p1:( p p1 value )p2:( p p1 children ) 
		]
	===========
	and answer will be a sequence containing a depth first traversal of all the
	values of all the nodes as a single sequence.
	This sequence is lazy.  It does not iterate over itself until you send it an
	iteration message like
	>>asOrderedCollection or >>do: or >>inject:into: or >>asString etc.  
	So when you send the
	inlineEmbeddedSequencesDeep:
	above you get back answer instantly no matter how big tree is.
	When you tell answer to iterate over tree that is when answer will do a lot
	of work as in-:
	----------------------
	( answer asOrderedCollection )
	===========
	etc.
	You have to be careful about the-:(
		p p1:( p p1 value )p2:( p p1 children )
	)
	part above.  The ( p p1 children ) part is important.  It has to dig some
	isSequence sequence S out of the-:( p p1 ) Object.  This sequence S can
	be nil or be empty.  That is okay.  But is has to be some sequence.  There
	can be nasty infinite loops that come up if this part is not done right.
	See the testCases below for more details about one case where isKindOf:
	is used to sort this kind of thing out.  Also see
	KEGAccountKeepTrackerPosition>>dependents for another example.
	You see the ( p p1 children ) part must return the children of the value.
	Because this is where the actual depth first traversal is done.  Right in
	this ( p p1 children ) expression.  Or it might otherwise be done as
	( p p1 asGenerator ) etc.  You need to take the p p1 Object and dig
	out the sub sequence inside of it that is to be the next level down in
	the traversal.  Because if this is not done then an infinite loop will
	get entered where the Object>>asStreamGeneratorOn: method just
	keeps getting called and recalled with the same contents and bang
	you've got an infinite loop.  The depth first traversal must be
	progressed down a level in the [ :p | ... ] block shown above.
	Or you get a nasty infinite loop that is hard to step into using the
	stepping debugger to figure out what went wrong.  So I am telling you
	now what went wrong because if you try to step into the Generator
	returned by self-:( this method ) to find out where and why there is
	an infinite loop there is a good chance that you won't be able to
	figure it out, until you have spent hours and hours on it.  So I am
	telling you now what the probable cause is.  And that you should
	look to your [ :p | ... ] block and make sure it is progressing the
	depth first traversal down a level, as shown in the examples.
	. . . . . . . . . . .
	You could also do some processing at each node as you are doing this
	by doing-:
	---------------------
	answer := ( tree asGenerator )
		inlineEmbeddedSequencesDeep:[ :p |
			p p1:( p p1 value process )p2:( p p1 children )
		]
	===========
	in which case you may not even care about the values that answer returns
	when you send it >>next over and over or iterate over it using one of the
	iteration methods.  You may just be interested in the
	side effects of the >>process message.  In which case you could just do-:
	-----------------
	( answer iterate )
	=========
	In the cases where >>process or >>children are nil there are speed
	optimizations and these nils don't show up in the answer sequence.  Usually
	if an upstream Generator hands it's owner Generator G a nil that stops
	G in its tracks and G will return nil from then on when it gets the >>next
	message.  Until G gets the >>reset or >>first messages that is .  Which restart
	G back to the beginning of G.  Where G is a sequence.
	Any iteration methods will stop and return what they have
	got up to that point and all Generators that recieved nil will return nil from
	then on until they get the >>reset message.  That isn't the case here.  If that
	kind of functionality is required here then you might try raising an
	exception( << See BOOK-: Advanced Smalltalk ISBN-:
	0-471-16350-3 >> ) or you could modify self( this method ) so that if
	( param at:2 ) above returns
	#stopInlineEmbeddedSequencesDeep: then the block parameter into
	>>atEachGenerate: above returns nil in which case answer and all
	Generators feeding off of it and all iteration method feeding
	off of them will be stopped.  But don't get the idea that everything will
	be stopped.  Just the Generators that see that their next element is a nil
	will be stopped( ie: from then on they too will return nil ).  ( All Generators
	are infinite in length.  But most of them have long tails of nothing but nils. )
	You could do this by changing-:
	
	atEachGenerate:[ :x |
		param := explodeBlock value: x .
		bla bla bla
	]
	atEachGenerate:[ :x |
		( 	( param := explodeBlock value: x ) 
			== 
			#stopInlineEmbeddedSequencesDeep: 
		)ifTrue:[
			bla bla bla
		]
	]
	And if you do make this change or any other changes which seem generally
	useful then you can send me the changed self( this method ) to
	picoVerse@gMail.com with subject 
	'PICOVERSE-: KEGGenerator>>inlineEmbeddedSequencesDeep: change' .

	BUT WAIT!!-:(
		You should never modify my Classes or methods.  As a matter of principle.
		You must do it as a last ditch effort after all other options have been
		tried.  In the picoVerse nobody modifies anybody else's methods or
		Classes.  You must either subClass or use whatever other flexibility ways
		I have provided.  And if you do make a change to one of my methods
		or anybody else's methods you must put such changes in a Package
		with a name like-:(
			JQPMadeDangerousChangesToKEGSystem
		)
		Or if you are making changes to Dolphin factory original methods
		or Classes you must put them into a Package called-:(
			JQPSystemChangesDangerous
		)
		where JQP stands for your initials.  So if your initials were JPM
		you would replace JQP above with JPM.
		And these Dangerous changes Packages should just have a few
		methods in them.  Less than 10 We would hope.
		You should absolutely try to make all the changes you need to make
		via the flexibility ways that the code owner has already provided.
		Or e-mail them with your changes so maybe they can patch them
		in if they want.  Using subject-:( 
			'( picoVerse-:( Suggested Changes to YOUR-: ... ) )'
		).
		But if they have made their thing right you should be able to make
		your changes without touching the originators code.  You should do all
		you can not to touch the originator's code.  That is the OOP way.
		So use it.  Do not touch the originator's code.
		And a note to you originators.  Make your code as general and flexible
		as you can.  Use all the tricks I use.  Like allowing KEGParams to
		be input into input parameters or allowing Blocks to be inputs.
		Or Objects that have special protocals that can be overridden.  I say
		again.  In the picoVerse we-:  Do not touch the originator's code.
		CVS be DAMNED.  We don't do that here.  And in this I am unanimous.
	)-butwait

	Suppose you just have a tree of sequences and not Nodes( the nodes are just
	sequences ).
	Where the sequences can actually be are any Objects not just sequences(
	most Objects can be considered to be sequences with just themselves in the
	sequence ).
	Often I say sequence and the idea is that  s is a sequence if
	( s isSequence ) = true
	But often I mean sequence informally. I am bending the true definition
	because all arbitrary Objects are sequences really because they all
	respond to asGenerator and all Generators are sequences.  But not all
	Objects cause isSeqence to return true, nor should they. 
	( String new isSequence ) = false
	And it is very important that all Strings NOT be sequences.
	The informal definition of sequence is any Object x which responds to
	>>asGenerator.  Which is all Objects.  So we have in mind that we have
	a real sequence s ( ( s isSequence ) = true ) but also
	that it could really be anything because in a pinch anything will still work.
	But what we had in mind when we built the thing was that it would mostly be
 	dealing with real isSequence sequences.  But it
	fails gracefully by assuming that all Objects are sequences.
	So you just want to iterate recursively over the sequences in this tree.  Then
	you could do-:
	------------------
	answer := ( tree asGenerator )	inlineEmbeddedSequencesDeep:( nil )
	==========
	Since nil = [ :p | p ] which causes the default action to always be
	executed.  ie: each element in the tree is either a sequence or a non sequence.
	And the non sequences are returned in the depth first order they apear in the
	tree.  Where the nodes in the tree are the sequences.  ie: The nodes have no
	values but they do have children.  So all the children are returned as a
	sequence in depth first order.
	or you could just do-:
	-------------------
	answer := ( tree asGenerator ) inlineEmbeddedSequencesDeep .
	==========
	which is cleaner as well as probably faster than-:(
		( tree asGenerator )inlineEmbeddedSequencesDeep:( nil )
	).





	GLOSSARY-:
	suppose we have a tree
	( 1 ( 2 3 ( 4 ( 5 6 7 ) ) 8 ( 9 ( 10 11 ) ) 12 13 14 ( 15 ) ))
	=
	1
		2
		3
			4
				5
				6
				7
		8
			9
				10
				11
		12
		13
		14
			15
	then a depth first traversal is a traversal of all the nodes( each node here is
	represented by a number )
	which visits each node in the depth first order which is 1 2 3 4 5 6 7 8 9 10 11
	12 13 14 15.
	All trees can be represented by such an indenting with numbers representing
	the nodes. ( all acyclic trees ).  Another example of a tree could be
	( 1 ( 2 ( 3 ( 4 ( 5 ) ) ) 6 7 ( 8 9 10 ) ) )
	=
	1
		2
			3
				4
					5
		6
		7
			8
			9
			10
	which has depth first traversal 1 2 3 4 5 6 7 8 9 10.
	If we just mix up the numbers like-:
	( 8 ( 1 ( 4 ( 7 ( 2 ) ) ) 10 9 ( 3 5 6 ) ) )
	=
	8
		1
			4
				7
					2
		10
		9
			3
			5
			6
	then the depth first traversal is 8 1 4 7 2 10 9 3 5 6 .
	Some of the nodes can have children.
	If you think of a sequence ( v c1 c2 c3 ... cN ) as being a node with value
	= v and children = ( c1 c2 c3 ... cN )
	Like in the above tree node 8 has one singe
	child-:( 
		( 1 ( 4 ( 7 ( 2 ) ) ) 10 9 ( 3 5 6 ) )
	)
	and node 1 has children-:(
		( 4 ( 7 ( 2 ) ) )
		10
		9
		( 3 5 6 )
	).
	
 )-detail
 usage-:(
	usage
 )-usage
 todos-:(
	todos
 )-todos
 changes-:(
	(| KEG2-:( Changes onDate-:( ... ) madeChanges-:( ... )-madechanges )-chg )-keg2
 )-changes
 testCases-:( "

	| p testBlk |

	( testBlk := [ :tree |
		( tree asGenerator ) inlineEmbeddedSequencesDeep:( nil ) 
	] ). 

	( self assert:(
	( Array
		with:
		(
		(
		( 
			testBlk value: #( (1 2 3) '(444)' (5) (6 7 8) 9 10 ) 
		) 	asArray
		)	= #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		)
		with:
		(
		(
		( 
			testBlk value: #( (1 2 3) '(444)' (5) ((6) 7 8) 9 10 ) 
		) 	asArray
		)	= #( 1 2 3 '(444)' 5 6 7 8 9 10 )
		)
		with:[

			( testBlk := [ :tree |
				( tree asGenerator ) inlineEmbeddedSequencesDeep:[ :param | 
					( ( p := param p1 ) isKindOf:( Point ) )
						ifTrue:[ param p1:( p x )p2:( p y ) ]"<---[ x is the value and y is the children ]"
						ifFalse:[ param ]"<-----------------------------[ accept the default processing of the Arrays and Strings and Numbers ]"
				] 
			] ).

			(
			(
			( 
				testBlk value: 
					( Array
						with:( 1@#(2 3 4) )"<---[ value=1 children=#( 2 3 4 ) ]"
						with:( 5@'(666)' )
						with:( 7@
							(	Array
								with:( 8@9 )
								with:( 10@#(11  12  13) )
								with:( 14@15 )
								with:( 16@17 )
							) 
							)
					)
			) 	asArray
			)	= #( 1 2 3 4 5 '(666)' 7 8 9 10 11 12 13 14 15 16 17 )
			)
		] value
	)= #( true true true )
	) description:( '' ) ).

 " )-testCases
 alternateCode-:(

			( p2 := ( param at:2 ) )isNil
				ifTrue:[ atEachGen := streamGen1 ]""<------------------------------------------------( We don't actually need this line but it gets rid of the concatGen when we don't need it.  Which might speed things up just a tiny bit. )""
				ifFalse:[ 
					( atEachGen := concatGen ).
					inlineGen := 
						( 
							( param at:2 )asStreamGeneratorOn:( streamGen2 on:( oc2 ) ) 
						)	inlineEmbeddedSequencesDeep:( explodeBlock )""<---------------( Here is where we set the contents of abstractGen2 to be the depth first traversal of the children of tree node x.  The >>on: message Reprimes the streamGen2 in case the previous time around the atEachGenerate: loop ( param at:2 ) was a Collection other than OrderedCollection.  We need OrderedCollection oc2 to be in there now as a speed optimization in case ( param at:2 ) is a String. )""
				]
			).

 )-alternateCode
 rants-:(
	rantsrantsrants rantsrants
 )-rants )-defineMethod"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:12'!
"(| defineMethod-:(" testrepeatC "repeat: anInteger "")
	notes-:(		Method Version: keg1.
				 Copyright Kjell E God 20030217
	licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details.
	Date of Last Revision: Date today tradePrint 20030217
	)-notes methodBody-:(
 (| let-:(""| returnFirst first last |"")
    letBody-:("					"( MT b:#repeat: s:self ).""
	^(
		( ( anInteger isKindOf:( Integer ) )
		)   ifTrue:[ ( 1 to: anInteger ) asGenerator atEachGenerate:[ :i | self ] ]ifFalse:[ 
		( ( self isOnAStream )and:[ self generator size = 2 ]
		)   ifTrue:[ 
			( self abstractGenerator new 
			) resetBlock:[
				( first := self first ).( last := self next ).( returnFirst:=false ) ]
			  nextBlock:[
				( returnFirst:=( returnFirst not ) )ifTrue:[ first ]ifFalse:[ last ] ]
		    ]
		ifFalse:[ 
			self isEmpty ifFalse:[ ( self asSourceGen )inlineEmbeddedSequences ] ] ]""<---[ Infinite recursion in inlineEmb... if self isEmpty ]""

	)""return:( 						MT e:#repeat: s:self )"

    ")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(
	If self = < s1 s2 s3 ... sN > then return a sequence r where
	r = <		s1 s2 s3 ... sN
			s1 s2 s3 ... sN
			s1 s2 s3 ... sN
			. . .
			s1 s2 s3 ... sN	>
	where self is repeated anInteger times.
	If anInteger is not a Number ( like if it is nil or #all or #infinity ) then
	r = <		s1 s2 s3 ... sN
			s1 s2 s3 ... sN
			s1 s2 s3 ... sN
			. . .				>
	is returned which is infinitely long if self-:( this computer ) can keep
	running forever.  Which probably can't happen since as far as we know
	now the time span in which some matter as we know it can exist in this
	universe is finite.

	If self is on a Stream of size 2 then a special optimization is done in that
	case which is used by the KEGGenerator class>>dualWriteStreams: method.

	If self is empty and anInteger = #infinite or some other non Integer thing that
	selects the infinite case then self This Method will return nil to avoid an
	infinite recursion in the >>inlineEmbeddedSequences Method.

 )-summary
 methodDescriptionDetail-:(

	In the case where self isOnAStream and is of size 2 a special speed
	optimization is done.  This size 2 setup can be useful when you are
	trying to make a Generator lazy that needs two of some data transfer
	setup to exist in memory at the same time and keep alternating and
	switching places for each Generator iteration-:(

	| writeGen nextWriteGen makeReadGen |
	( makeReadGen := KEGGenerator streamGenerator ).
	( writeGen := ( Array new:( 100 ) ) readWriteStream asGenerator ).
	( "".......................................""
	(
	(
	(
	(	#( 	( 6 7 8 )
			( 9 10 11 ) 
			( 12 13 14 )
			( 15 16 17 )
			( 18 19 20 )
			( 21 22 23 )
			( 24 25 26 )
			( 27 28 29 )
			( 30 31 32 )
			( 33 34 35 ) ) asGenerator
	) collect:[ :array | makeReadGen on:( array );generator ]
	) first:[ :readStream |
		( nextWriteGen :=
		(
		( 										Array p1:
		    [ readStream ]							p2:
		    [ writeGen nextPutAll:( readStream contentsKEG ) ;
			generator ] ""<---( returns the ReadWriteStream above )""
		) asGenerator repeat:( #asSourceGen )
		) reset;yourself
		) ] 
	  next:[ :ws | nextWriteGen ]
	) collect:[ :nextBlkGen | nextBlkGen next value ]
	) alternateWith:#( 1 2 3 4 5 )
	)

	)-iteration
	which as of <D:20030425> has never been tested.
	But it makes a nextWriteGen which is of the form-:( Let
		s1=[ readStream ]
		s2=[ writeGen nextPutAll:( readStream contentsKEG ) ; ... ]
		< s1 s2 s1 s2 s1 s2 s1 s2 s1 s2 s1 s2 s1 s2 ... >
	) in other words it is a sequence of two BlockClosures that alternate
	with each other forever and each BlockClosure returns a different
	WriteStream.  Such that when alternateWith: requests the second
	WriteStream that it needs it won't get the first one over again which
	would screw things up.  Thus only two WriteStreams need to be
	allocated.  Instead of 10 or the number of subArrays of the top
	Array which could be arbitrarily large as would be the case in the
	Collections only implementation with no Generator lazyness.
	So once the Generator returned by-:(
		collect:[ :nextBlkGen | nextBlkGen next value ]
	)above gets going it should go pretty quick as the >>next is highly
	optimized.  As you can see from the code above in the body.
	But it should be noted that there is probably a faster way than
	using >>first:next: which is not slow.  But still and all.

 )-detail
 usage-:(
	usage
 )-usage
 todos-:(
	todos
 )-todos
 changes-:(
	(| KEG2-:( Changes onDate-:( ... ) madeChanges-:( ... )-madechanges )-chg )-keg2
 )-changes
 testCases-:( "

	( self assert:( 
		( ( #(  ) asGenerator repeat: 1 ) asArray = #(  ) )
		) description:( '( ( #(  ) asGenerator repeat: 1 ) asArray = #(  ) )' ) ).

	( self assert:( 
		( ( #(  ) asGenerator repeat: 0 ) asArray = #(  ) )
		) description:( '( ( #(  ) asGenerator repeat: 0 ) asArray = #(  ) )' ) ).

	( self assert:( 
		( ( #( 1 ) asGenerator repeat: 0 ) asArray = #(  ) )
		) description:( '( ( #( 1 ) asGenerator repeat: 0 ) asArray = #(  ) )' ) ).

	( self assert:( 
		( ( #( 1 ) asGenerator repeat: 1 ) asArray = #( 1 ) )
		) description:( '( ( #( 1 ) asGenerator repeat: 1 ) asArray = #( 1 ) )' ) ).

	( self assert:( 
		( ( #( 1 ) asGenerator repeat: 2 ) asArray = #( 1 1 ) )
		) description:( '( ( #( 1 ) asGenerator repeat: 2 ) asArray = #( 1 1 ) )' ) ).

	( self assert:( 
		( ( #( 1 ) asGenerator repeat: 3 ) asArray = #( 1 1 1 ) )
		) description:( '( ( #( 1 ) asGenerator repeat: 3 ) asArray = #( 1 1 1 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 ) asGenerator repeat: 0 ) asArray = #(  ) )
		) description:( '( ( #( 1 2 ) asGenerator repeat: 0 ) asArray = #(  ) )' ) ).

	( self assert:( 
		( ( #( 1 2 ) asGenerator repeat: 1 ) asArray = #( 1 2 ) )
		) description:( '( ( #( 1 2 ) asGenerator repeat: 1 ) asArray = #( 1 2 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 ) asGenerator repeat: 2 ) asArray = #( 1 2 1 2 ) )
		) description:( '( ( #( 1 2 ) asGenerator repeat: 2 ) asArray = #( 1 2 1 2 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 ) asGenerator repeat: 3 ) asArray = #( 1 2 1 2 1 2 ) )
		) description:( '( ( #( 1 2 ) asGenerator repeat: 3 ) asArray = #( 1 2 1 2 1 2 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 3 ) asGenerator repeat: 0 ) asArray = #(  ) )
		) description:( '( ( #( 1 2 3 ) asGenerator repeat: 0 ) asArray = #(  ) )' ) ).

	( self assert:( 
		( ( #( 1 2 3 ) asGenerator repeat: 1 ) asArray = #( 1 2 3 ) )
		) description:( '( ( #( 1 2 3 ) asGenerator repeat: 1 ) asArray = #( 1 2 3 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 3 ) asGenerator repeat: 2 ) asArray = #( 1 2 3 1 2 3 ) )
		) description:( '( ( #( 1 2 3 ) asGenerator repeat: 2 ) asArray = #( 1 2 3 1 2 3 ) )' ) ).

	( self assert:( 
		( ( #( 1 2 3 ) asGenerator repeat: 3 ) asArray = #( 1 2 3 1 2 3 1 2 3 ) )
		) description:( '( ( #( 1 2 3 ) asGenerator repeat: 3 ) asArray = #( 1 2 3 1 2 3 1 2 3 ) )' ) ).

	( self assert:( 
		( ( #(  ) asGenerator repeat: #infinite ) = nil )
		) description:( '( ( #(  ) asGenerator repeat: #infinite ) = nil )' ) ).

	( self assert:( 
		( ( ( #( 1 ) asGenerator repeat: #infinite ) first: 3 ) asArray = #( 1 1 1 ) )
		) description:( '( ( ( #( 1 ) asGenerator repeat: #infinite ) first: 3 ) asArray = #( 1 1 1 ) )' ) ).

	( self assert:( 
		( ( ( #( 1 2 ) asGenerator repeat: #infinite ) first: 6 ) asArray = #( 1 2 1 2 1 2 ) ) "<---[ Special optimized case ]"
		) description:( '( ( ( #( 1 2 ) asGenerator repeat: #infinite ) first: 6 ) asArray = #( 1 2 1 2 1 2 ) )' ) ).

	( self assert:( 
		( ( ( #( 1 2 3 ) asGenerator repeat: #infinite ) first: 9 ) asArray = #( 1 2 3 1 2 3 1 2 3 ) )
		) description:( '( ( ( #( 1 2 3 ) asGenerator repeat: #infinite ) first: 9 ) asArray = #( 1 2 3 1 2 3 1 2 3 ) )' ) ).

 " )-testCases
 alternateCode-:(
 )-alternateCode
 designs-:(
	( DESIGN subject-:( aPackageName )of-:(
	)-of )-design
 )-designs
 rants-:(









Would you keep your pants on.

 . . .

oh..- . . .     :			i mean :

			           - sir.











For I have been no idle hazzard of empty idle hand,
But have succeeded in even a blind valor of full unknowing distraction
Even with full blown fell and blasted Poverty of mind, 
As Sums to Not One single active recorded V checking thought
Have blown up All fairest sweetest suites by such metalic disttractive
Blinders on Ignore ance as test now torch cutting white and hot for
Sore MissLaid occassion's fairest rainy carriage  white :
						Lost, All lost, and quite at sea
But unknown blind then unthrown unlooked at dates
On unseen burning pyres of still    MORE    now terrible reverie
With such impress of wordwright madness whose sore prefix'ed
And pre determinate of 4 weeks on
Grinding decendent fell terrible task that
Swiftly to cliffs of mettle exhaustion speed
Throwing away all, And drawing up to   collection  all in all,
Doth not divide the Sunday from the week
That be toward: In the end: Alas: A counterfit Candies dissoccassion 
In but her mind's own eye,  Coming found, sad, most sad, After all,
And now Imploring pardon, That this sweaty Haste
Doth make the night compounded-laborer with the day
Its own now Highest mart unwilling to stay,
For allowance of any other action
Save the odd embedded seek

So just be patient.    you.        Okay...











-best wishes












( elapse time 1 hour )( non repeatable )
( POSITION-: 212@0 EXTENT-: 751@780 IN-: 1024@768 )
( FONT-: Times New Roman SIZE-: 14 BOLD-: on )









 )-rants )-defineMethod"! !

!KEGGeneratorTest methodsFor: '*KEGGeneratorInliningTest' stamp: 'KjellGodo 5/9/2013 20:23'!
"(| defineMethod-:(" testseparatedByC "separatedBy: aSequence "")
	notes-:(		Method Version: keg4.
				 Copyright Kjell E God 20110521
	licensed as MIT Open Source: Please execute ( Object licenseMIT ) for details.
	Date of Last Revision: Date today tradePrint ( 20110521 13:56 )
	)-notes methodBody-:(
 (| let-:(""| seperators |"")
    letBody-:(""					""( MT b:#separatedBy: s:self ).""
	( seperators := aSequence asGenerator repeatInfinite ).
	^(	(
		( self reset:[ :na | ( self reset ).( seperators reset ) ]
		) collect:[ :x | x ]separatedBy:[ :x | seperators next ] 
		)

	)""return:( 					MT e:#separatedBy: s:self )"

    ")-letbod )-let
 )-methBod
 methodDescriptionSummary-:(

	Return a new sequence based on self with seperator Objects
	inserted between the elements of self.  Just like the standard ANSI
	Collection based >>collect:separatedBy: .But the seperator Objects
	come from aSequence which is converted to a repeatInfinite sequence
	such that aSequence is repeated an infinite number of times until self
	returns nil by the self>>next message( ie: when self ends ).

 )-summary
 methodDescriptionDetail-:(

	PUBLIC-: Return a Generator with the following structure-:
	Let self = s = <s1 s2 s3... sN-1 sN>.
	Let aSequence = sq = <sq1 sq2 sq3...sqM >.
	Then seperators = sp = < sq1 sq2 sq3 ... sqM sq1 sq2 sq3 ... sqM sq1 sq2 sq3 ... >
		Let sp = < sp1 sp2 sp3 ... > trivially by renaming the elements of sp.
	If aSequence is infinite then seperators = < sq1 sq2 sq3 ... >
	Let the returned Generator = r then r =
		<s1 sp1 s2 sp2 s3 sp3 ... spQ sN>
	where spQ is some element in the sp sequence but we don't know which although
	we could figure it out if we wanted to worry.

	(( a sequence is any Object in some variable-:( s )that causes
	   the expression ( s isSequence ) to return-:( true )when it is evaluated.
	   All KEGGenerators will cause ( s isSequence ) to return-:( true )when
	   they are put into the some Smalltalk variable-:(  s ) and then ( s isSequence )
	   is evaluated.

 )-detail
 usage-:(
	usage
 )-usage
 todos-:(
	todos
 )-todos
 changes-:(
	(| KEG2-:( Changes onDate-:(  ) madeChanges-:( 	)-madechanges )-chg )-keg2
 )-changes
 testCases-:( "

	( self assert:(
		( ( ( #(  ) asGenerator ) separatedBy:#( '-' ) ) asArray = #(  ) ) 
		) description:( '( ( ( #(  ) asGenerator ) separatedBy:#( `-` ) ) asArray = #(  ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 ) asGenerator ) separatedBy:#( '-' ) ) asArray = #( 1 ) ) 
		) description:( '( ( ( #( 1 ) asGenerator ) separatedBy:#( `-` ) ) asArray = #( 1 ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 2 ) asGenerator ) separatedBy:#( '-' ) ) asArray = #( 1 '-' 2 ) ) 
		) description:( '( ( ( #( 1 2 ) asGenerator ) separatedBy:#( `-` ) ) asArray = #( 1 `-` 2 ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 2 3 4 ) asGenerator ) separatedBy:#( '-' ) ) asArray = #( 1 '-' 2 '-' 3 '-' 4 ) ) 
		) description:( '( ( ( #( 1 2 3 4 ) asGenerator ) separatedBy:#( `-` ) ) asArray = #( 1 `-` 2 `-` 3 `-` 4 ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 3 5 ) asGenerator ) separatedBy:#( 2 4 ) ) asArray = #( 1 2 3 4 5 ) ) 
		) description:( '( ( ( #( 1 3 5 ) asGenerator ) separatedBy:#( 2 4 ) ) asArray = #( 1 2 3 4 5 ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 3 ) asGenerator ) separatedBy:#( 2 4 ) ) asOrderedCollection asArray = #( 1 2 3 ) ) 
		) description:( '( ( ( #( 1 3 ) asGenerator ) separatedBy:#( 2 4 ) ) asOrderedCollection asArray = #( 1 2 3 ) ) ' ) ).

	( self assert:(
		( ( ( #( 1 3 5 7 ) asGenerator  ) separatedBy:#( 2 4 ) ) asOrderedCollection asArray = #( 1 2 3 4 5 2 7 ) )
		) description:( '( ( ( #( 1 3 5 7 ) asGenerator  ) separatedBy:#( 2 4 ) ) asOrderedCollection asArray = #( 1 2 3 4 5 2 7 ) )' ) ).

 " )-testCases
 alternateCode-:(
	alternateCode
 )-alternateCode
 designs-:(
	( DESIGN subject-:( aPackageName )of-:(
	)-of )-design
 )-designs
 rants-:(
	rants
 )-rants )-defineMethod"! !
